#!/usr/bin/python
#
# Copyright (c) Andrea Micheloni 2011
#
# mDNS/DNS-SD tracker for the lazy social engineer
#
#       This program is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sqlite3

class DataStore:
	def __init__(self):
		self.connection = sqlite3.connect("eye-datastore.sqlite")
		try:
			self.initStore()
		except sqlite3.OperationalError:
			pass

	def initStore(self):
		self.connection.execute("CREATE TABLE Types (id INTEGER PRIMARY KEY AUTOINCREMENT, type VARCHAR UNIQUE)")
		self.connection.execute("CREATE TABLE Services (id INTEGER PRIMARY KEY AUTOINCREMENT, type INTEGER, name VARCHAR)")
		self.connection.execute("CREATE TABLE ServiceLogs (id INTEGER PRIMARY KEY AUTOINCREMENT, service INTEGER, datetime DATETIME, appeared BOOLEAN)")
		self.connection.execute("CREATE TABLE ResolutionAppearance (appearance INTEGER, resolution INTEGER, address VARCHAR, port INTEGER, PRIMARY KEY(appearance, resolution))")
		self.connection.execute("CREATE TABLE ResolutionConstant (id INTEGER PRIMARY KEY AUTOINCREMENT, host VARCHAR, txt VARCHAR, UNIQUE(host, txt))")
		self.connection.commit()

	def storeType(self, stype):
		cursor = self.connection.cursor()
		cursor.execute("SELECT id FROM Types WHERE Types.type=?;", (stype,))
		id=cursor.fetchone()
		
		if id == None:
			cursor.execute("INSERT INTO Types (type) VALUES (?)", (stype,))
			cursor.execute("SELECT last_insert_rowid();")
			id=cursor.fetchone()
			self.connection.commit()

		cursor.close()
		return id[0]

	def storeService(self, stype_id, name):
		cursor = self.connection.cursor()
		cursor.execute("SELECT id FROM Services WHERE Services.type=? AND Services.name=?;", (stype_id, name))
		id=cursor.fetchone()
		
		if id == None:
			cursor.execute("INSERT INTO Services (type, name) VALUES (?,?)", (stype_id, name))
			cursor.execute("SELECT last_insert_rowid();")
			id=cursor.fetchone()
			self.connection.commit()

		cursor.close()
		return id[0]

	def storeServiceAppearance(self, service_id, appearance):
		if appearance:
			appeared="TRUE"
		else:
			appeared="FALSE"

		cursor = self.connection.cursor()
		cursor.execute("INSERT INTO ServiceLogs (service, appeared, datetime) VALUES (?,?,datetime('now'))", (service_id, appeared))
		cursor.execute("SELECT last_insert_rowid();")
		id=cursor.fetchone()
		self.connection.commit()

		cursor.close()
		return id[0]

	def storeServiceResolution(self, appearance_id, host, address, port, txt):
		cursor = self.connection.cursor()
		cursor.execute("SELECT id FROM ResolutionConstant WHERE ResolutionConstant.host=? AND ResolutionConstant.txt=?;", (host, txt))
		resolution_id=cursor.fetchone()
		
		if resolution_id == None:
			cursor.execute("INSERT INTO ResolutionConstant (host, txt) VALUES (?, ?)", (host, txt))
			cursor.execute("SELECT last_insert_rowid();")
			resolution_id=cursor.fetchone()
			self.connection.commit()
		resolution_id = resolution_id[0]

		cursor.execute("INSERT INTO ResolutionAppearance (appearance, resolution, address, port) VALUES (?,?,?,?)", (appearance_id, resolution_id, address, port))
		cursor.execute("SELECT last_insert_rowid();")
		id=cursor.fetchone()
		self.connection.commit()

		cursor.close()
		return id[0]
